#include "qwayland-server-share-buffer.h"

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    qt_share_buffer::qt_share_buffer(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_share_buffer::qt_share_buffer(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_share_buffer::qt_share_buffer()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_share_buffer::~qt_share_buffer()
    {
    }

    void qt_share_buffer::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_share_buffer::Resource *qt_share_buffer::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_share_buffer::Resource *qt_share_buffer::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_share_buffer::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_share_buffer_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_share_buffer::Resource *qt_share_buffer::share_buffer_allocate()
    {
        return new Resource;
    }

    void qt_share_buffer::share_buffer_bind_resource(Resource *)
    {
    }

    void qt_share_buffer::share_buffer_destroy_resource(Resource *)
    {
    }

    void qt_share_buffer::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_share_buffer *that = static_cast<qt_share_buffer *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_share_buffer::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_share_buffer *that = resource->share_buffer_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->share_buffer_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_share_buffer::Resource *qt_share_buffer::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = share_buffer_allocate();
        resource->share_buffer_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_share_buffer_interface, version, id);
        wl_resource_set_implementation(handle, 0, resource, destroy_func);
        resource->handle = handle;
        share_buffer_bind_resource(resource);
        return resource;
    }

    void qt_share_buffer::send_cross_buffer(struct ::wl_resource *buffer)
    {
        send_cross_buffer(
            m_resource->handle,
            buffer);
    }

    void qt_share_buffer::send_cross_buffer(struct ::wl_resource *resource, struct ::wl_resource *buffer)
    {
        qt_share_buffer_send_cross_buffer(
            resource,
            buffer);
    }

}

QT_END_NAMESPACE
